@@ -22,6 +22,8 @@ import cravat.admin_util as au
 import re
 from types import SimpleNamespace
 import cravat.util
+from distutils.version import LooseVersion
+from cravat.constants import cannonical_chroms
 
 class BaseAnnotator(object):
 
@@ -84,6 +86,7 @@ class BaseAnnotator(object):
                 self.annotator_version = self.conf['version']
             else:
                 self.annotator_version = ''
+            self.supported_chroms = set(cannonical_chroms)
         except Exception as e:
             self._log_exception(e)
 
@@ -205,6 +208,11 @@ class BaseAnnotator(object):
                 print('        {}: started at {}'.format(self.module_name, time.asctime(time.localtime(start_time))))
             self.base_setup()
             last_status_update_time = time.time()
+            output_columns = self.conf['output_columns']
+            json_colnames = []
+            for col in output_columns:
+                if 'table' in col and col['table'] == True:
+                    json_colnames.append(col['name'])
             for lnum, line, input_data, secondary_data in self._get_input():
                 try:
                     if self.update_status_json_flag and self.status_writer is not None:
@@ -212,14 +220,20 @@ class BaseAnnotator(object):
                         if lnum % 10000 == 0 or cur_time - last_status_update_time > 3:
                             self.status_writer.queue_status_update('status', 'Running {} ({}): line {}'.format(self.conf['title'], self.module_name, lnum))
                             last_status_update_time = cur_time
-                    if input_data.get('alt_base', '') == '*': # VCF format * allele gets empty annotation.
-                        output_dict = {}
+                    # VCF format * allele gets empty annotation.
+                    if self.conf['level']=='variant' and input_data.get('alt_base', '') == '*': 
+                        output_dict = None
+                    # Skip alt chroms by default
+                    elif self.conf['level']=='variant' and not input_data.get('chrom') in self.supported_chroms:
+                        output_dict = None
                     elif secondary_data == {}:
                         output_dict = self.annotate(input_data)
                     else:
                         output_dict = self.annotate(input_data, secondary_data)
+                    for colname in json_colnames:
+                        output_dict[colname] = json.dumps(output_dict[colname])
                     # This enables summarizing without writing for now.
-                    if output_dict == None:
+                    if output_dict is None:
                         continue
                     # Preserves the first column
                     output_dict[self._id_col_name] = input_data[self._id_col_name]
@@ -236,10 +250,12 @@ class BaseAnnotator(object):
             self.base_cleanup()
             end_time = time.time()
             self.logger.info('finished: {0}'.format(time.asctime(time.localtime(end_time))))
-            print('        {}: finished at {}'.format(self.module_name, time.asctime(time.localtime(end_time))))
+            if self.args.silent == False:
+                print('        {}: finished at {}'.format(self.module_name, time.asctime(time.localtime(end_time))))
             run_time = end_time - start_time
             self.logger.info('runtime: {0:0.3f}s'.format(run_time))
-            print('        {}: runtime {:0.3f}s'.format(self.module_name, run_time))
+            if self.args.silent == False:
+                print('        {}: runtime {:0.3f}s'.format(self.module_name, run_time))
             if self.update_status_json_flag and self.status_writer is not None:
                 version = self.conf.get('version', 'unknown')
                 self.status_writer.queue_status_update('status', 'Finished {} ({})'.format(self.conf['title'], self.module_name))
@@ -258,10 +274,10 @@ class BaseAnnotator(object):
     async def get_gene_summary_data (self, cf):
         #print('            {}: getting gene summary data'.format(self.module_name))
         t = time.time()
+        module_ver = await cf.exec_db(cf.get_module_version_in_job, self.module_name)
         hugos = await cf.exec_db(cf.get_filtered_hugo_list)
-        cols = [self.module_name + '__' + coldef['name'] \
-            for coldef in self.conf['output_columns'] \
-            if coldef['name'] != 'uid']
+        output_columns = await cf.exec_db(cf.get_stored_output_columns, self.module_name)
+        cols = [self.module_name + '__' + coldef['name'] for coldef in output_columns if coldef['name'] != 'uid']
         data = {}
         t = time.time()
         rows = await cf.exec_db(cf.get_variant_data_for_cols, cols)
@@ -305,6 +321,8 @@ class BaseAnnotator(object):
             self._setup_outputs()
             self._open_db_connection()
             self.setup()
+            if not hasattr(self, 'supported_chroms'):
+                self.supported_chroms = set(['chr'+str(n) for n in range(1,23)]+['chrX','chrY'])
         except Exception as e:
             self._log_exception(e)
 
@@ -525,10 +543,7 @@ class BaseAnnotator(object):
 
 
 class SecondaryInputFetcher():
-    def __init__(self,
-                 input_path,
-                 key_col,
-                 fetch_cols=[]):
+    def __init__(self, input_path, key_col, fetch_cols=[]):
         self.key_col = key_col
         self.input_path = input_path
         self.input_reader = CravatReader(self.input_path)
@@ -554,9 +569,15 @@ class SecondaryInputFetcher():
             key_data = all_col_data[self.key_col]
             if key_data not in self.data: self.data[key_data] = []
             fetch_col_data = {}
+            row_has_data = False
             for col in self.fetch_cols:
+                if col!=self.key_col and all_col_data[col] is not None:
+                    row_has_data = True
                 fetch_col_data[col] = all_col_data[col]
-            self.data[key_data].append(fetch_col_data)
+            if row_has_data:
+                self.data[key_data].append(fetch_col_data)
+            # self.data[key_data].append(fetch_col_data)
+
 
     def get(self, key_data):
         if key_data in self.data:
