@@ -245,6 +245,7 @@ base_smartfilters = [
 ]
 
 module_tag_desc = {
+    'allele frequency': 'modules for studying allele frequency across populations',
     'cancer': 'tools for cancer research',
     'clinical relevance': 'tools for assessing clinical relevance of variants',
     'converters': 'modules for using the result of other tools as open-cravat input',
@@ -253,12 +254,10 @@ module_tag_desc = {
     'evolution': 'modules for studying variants in evolutionary context',
     'genes': 'modules for studying variants at the gene level',
     'genomic features': 'modules for studying genomic features',
-    'germline': 'modules for studying germline variants',
     'interaction': 'modules for studying molecular interactions',
     'literature': 'modules for variant-related literature',
     'multiple assays': 'modules for multiplex assays',
-    'noncoding': 'modules for studying noncoding variants',
-    'populations': 'modules for studying population statistics of variants',
+    'non coding': 'modules for studying noncoding variants',
     'protein visualization': 'modules to visualize variants on protein structures',
     'reporters': 'modules for generating output formats',
     'variant effect prediction': 'modules to predict variant effects',
@@ -277,4 +276,6 @@ default_assembly_key = 'default_assembly'
 assembly_choices = ['hg38', 'hg19', 'hg18']
 publish_time_fmt = '%Y-%m-%dT%H:%M:%S.%f%z'
 
-install_tempdir_name = 'temp'
\ No newline at end of file
+install_tempdir_name = 'temp'
+
+cannonical_chroms = ['chr'+str(n) for n in range(1,23)]+['chrX','chrY']
\ No newline at end of file
