@@ -81,6 +81,7 @@ class MasterCravatConverter(object):
         self._setup_logger()
         self.vtracker = VTracker(deduplicate=not(self.unique_variants))
         self.wgsreader = cravat.get_wgs_reader(assembly='hg38')
+        self.crs_def = constants.crs_def.copy()
 
     def _parse_cmd_args(self, inargs, inkwargs):
         """ Parse the arguments in sys.argv """
@@ -264,14 +265,7 @@ class MasterCravatConverter(object):
                     print(f'\n{msg}')
                     raise ExpectedException(msg)
         self.primary_converter = self.converters[self.input_format]
-        self.primary_converter.output_dir = self.output_dir
-        self.primary_converter.run_name = self.output_base_fname
-        self.primary_converter.input_assembly = self.input_assembly
-        module_name = self.primary_converter.format_name + '-converter'
-        if module_name in self.conf:
-            if hasattr(self.primary_converter, 'conf') == False:
-                self.primary_converter.conf = {}
-            self.primary_converter.conf.update(self.conf[module_name])
+        self._set_converter_properties(self.primary_converter)
         if self.pipeinput == False:
             if len(self.input_files) > 1:
                 for f in self.input_files[1:]:
@@ -281,6 +275,16 @@ class MasterCravatConverter(object):
                         f.seek(0)
         self.logger.info('input format: %s' %self.input_format)
 
+    def _set_converter_properties(self, converter):
+        converter.output_dir = self.output_dir
+        converter.run_name = self.output_base_fname
+        converter.input_assembly = self.input_assembly
+        module_name = self.primary_converter.format_name + '-converter'
+        if module_name in self.conf:
+            if hasattr(converter, 'conf') == False:
+                converter.conf = {}
+            converter.conf.update(self.conf[module_name])
+
     def _open_output_files (self):
         """ Open .crv .crs and .crm output files, plus .err file.
             
@@ -312,10 +316,10 @@ class MasterCravatConverter(object):
         # Setup crs sample file
         self.crs_path = os.path.join(self.output_dir, self.output_base_fname +'.crs')
         self.crs_writer = CravatWriter(self.crs_path)
-        self.crs_writer.add_columns(constants.crs_def)
+        self.crs_writer.add_columns(self.crs_def)
         if hasattr(self.primary_converter, 'addl_cols'):
             self.crs_writer.add_columns(self.primary_converter.addl_cols, append=True)
-            constants.crs_def.extend(self.primary_converter.addl_cols)
+            self.crs_def.extend(self.primary_converter.addl_cols)
         self.crs_writer.write_definition()
         for index_columns in constants.crs_idx:
             self.crs_writer.add_index(index_columns)
@@ -350,7 +354,9 @@ class MasterCravatConverter(object):
             else:
                 fname = f.name
             fileno += 1
-            self.primary_converter.setup(f)
+            converter = self.primary_converter.__class__()
+            self._set_converter_properties(converter)
+            converter.setup(f)
             if self.pipeinput == False:
                 f.seek(0)
             read_lnum = 0
@@ -366,7 +372,7 @@ class MasterCravatConverter(object):
                     # all_wdicts is a list, since one input line can become
                     # multiple output lines. False is returned if converter
                     # decides line is not an input line.
-                    all_wdicts = self.primary_converter.convert_line(l)
+                    all_wdicts = converter.convert_line(l)
                     if all_wdicts is BaseConverter.IGNORE:
                         continue
                     total_lnum += 1
@@ -413,7 +419,7 @@ class MasterCravatConverter(object):
                                         self.crl_writer.write_data(prelift_wdict)
                                     # addl_operation errors shouldnt prevent variant from writing
                                     try:
-                                        self.primary_converter.addl_operation_for_unique_variant(wdict, no_unique_var)
+                                        converter.addl_operation_for_unique_variant(wdict, no_unique_var)
                                     except Exception as e:
                                         self._log_conversion_error(read_lnum, l, e)
                                     no_unique_var += 1
