@@ -174,15 +174,27 @@ class CravatFilter ():
     #    for conn in self.open_conns.values():
     #        await conn.close()
 
+    async def get_module_version_in_job (self, module_name, conn=None, cursor=None):
+        if conn is None or cursor is None:
+            return None
+        q = 'select colval from info where colkey="_annotators"'
+        await cursor.execute(q)
+        anno_vers = await cursor.fetchone()
+        if anno_vers is None:
+            return None
+        version = None
+        for anno_ver in anno_vers[0].split(','):
+            [mname, ver] = anno_ver.split(':')
+            if mname == module_name:
+                version = ver
+                break
+        return version
+
     async def exec_db (self, func, *args, **kwargs):
         conn = await self.get_db_conn()
         cursor = await conn.cursor()
-        #conn_hash = conn.__hash__()
-        #self.open_conns[conn_hash] = conn
         ret = await func(*args, conn=conn, cursor=cursor, **kwargs)
-        #await cursor.close()
-        #await conn.close()
-        #del self.open_conns[conn_hash]
+        await cursor.close()
         return ret
 
     async def second_init (self):
@@ -288,7 +300,6 @@ class CravatFilter ():
             self.dbpath = dbpath
         conn = await self.get_db_conn()
         await conn.create_function('regexp', 2, regexp)
-        #await conn.close()
         await self.exec_db(self.set_aliases)
 
     async def set_aliases (self, conn=None, cursor=None):
@@ -313,8 +324,7 @@ class CravatFilter ():
         await conn.commit()
 
     async def filtertable_exists (self, conn=None, cursor=None):
-        sql = 'select name from sqlite_master where ' +\
-            'type="table" and name="' + self.filtertable + '"'
+        sql = f'select * from viewersetup where datatype="filter" and name="{self.filtername}"'
         await cursor.execute(sql)
         row = await cursor.fetchone()
         if row == None:
@@ -339,11 +349,11 @@ class CravatFilter ():
             self.filterstring = self.filterstring.replace("'", '"')
             self.filter = json.loads(self.filterstring)
         elif self.filtername is not None and filter_table_present:
-            await cursor.execute('select criteria from ' + self.filtertable +
-                ' where name="' + self.filtername + '"')
+            await cursor.execute('select viewersetup from viewersetup' +\
+                ' where datatype="filter" and name="' + self.filtername + '"')
             criteria = await cursor.fetchone()
             if criteria != None:
-                self.filter = json.loads(criteria[0])
+                self.filter = json.loads(criteria[0])['filterSet']
         elif self.filterpath is not None and os.path.exists(self.filterpath):
             with open(self.filterpath) as f:
                 ftype = self.filterpath.split('.')[-1]
@@ -739,6 +749,16 @@ class CravatFilter ():
             table_names.append(row[0].replace('_header', ''))
         return table_names
 
+    async def get_stored_output_columns (self, module_name, conn=None, cursor=None):
+        q = f'select col_def from variant_header where col_name like "{module_name}__%"'
+        await cursor.execute(q)
+        output_columns = []
+        for row in await cursor.fetchall():
+            d = json.loads(row[0])
+            d['name'] = d['name'].replace(f'{module_name}__', '')
+            output_columns.append(d)
+        return output_columns
+
 def regexp (y, x, search=re.search):
     if x is None:
         return 0
