@@ -41,6 +41,12 @@ class CravatReport:
         self.column_sub_allow_partial_match = {}
         self.colname_conversion = {}
         self.warning_msgs = []
+        self.colnames_to_display = {}
+        self.colnos_to_display = {}
+        self.display_select_columns = {}
+        self.extracted_cols = {}
+        self.conn = None
+        self.levels_to_write = None
         self.parse_cmd_args(inargs, inkwargs)
         global parser
         for ag in parser._action_groups:
@@ -57,19 +63,7 @@ class CravatReport:
         self.filterpath = parsed_args.filterpath
         self.filtername = parsed_args.filtername
         self.filterstring = parsed_args.filterstring
-        self.confs = None
-        if hasattr(parsed_args, 'conf') and parsed_args.conf is not None:
-            self.confs = parsed_args.conf
-        if parsed_args.confs is not None:
-            confs = parsed_args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
-            if self.confs is None:
-                self.confs = json.loads(confs)
-            else:
-                self.confs.update(json.loads(confs))
-        if self.confs is not None and 'filter' in self.confs:
-            self.filter = self.confs['filter']
-        else:
-            self.filter = None
+        self.confs = {}
         if parsed_args.output_dir is not None:
             self.output_dir = parsed_args.output_dir
         else:
@@ -80,12 +74,26 @@ class CravatReport:
         self.confpath = parsed_args.confpath
         self.conf = ConfigLoader(job_conf_path=self.confpath)
         self.module_name = parsed_args.module_name
+        if self.module_name in self.conf._all:
+            self.confs.update(self.conf._all[self.module_name])
         if self.conf is not None:
             self.module_conf = self.conf.get_module_conf(self.module_name)
         else:
             self.module_conf = None
         if hasattr(parsed_args, 'reporttypes'):
             self.report_types = parsed_args.reporttypes
+        if hasattr(parsed_args, 'conf') and parsed_args.conf is not None:
+            self.confs.update(parsed_args.conf)
+        if parsed_args.confs is not None:
+            confs = parsed_args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
+            if self.confs is None:
+                self.confs = json.loads(confs)
+            else:
+                self.confs.update(json.loads(confs))
+        if self.confs is not None and 'filter' in self.confs:
+            self.filter = self.confs['filter']
+        else:
+            self.filter = None
         self.output_basename = os.path.basename(self.dbpath)[:-7]
         status_fname = '{}.status.json'.format(self.output_basename)
         self.status_fpath = os.path.join(self.output_dir, status_fname)
@@ -112,8 +120,18 @@ class CravatReport:
             self.status_writer = parsed_args.status_writer
         else:
             self.status_writer = None
+        self.concise_report = parsed_args.concise_report
+        self.extract_columns_multilevel = self.get_standardized_module_option(self.confs.get('extract-columns', {}))
         self.args = parsed_args
 
+    def should_write_level (self, level):
+        if self.levels_to_write is None:
+            return True
+        elif level in self.levels_to_write:
+            return True
+        else:
+            return False
+
     async def prep (self):
         await self.connect_db()
         await self.load_filter()
@@ -128,6 +146,24 @@ class CravatReport:
         self.error_logger = logging.getLogger('error.' + self.module_name)
         self.unique_excs = []
 
+    async def get_db_conn (self):
+        if self.dbpath is None:
+            return None
+        if self.conn is None:
+            self.conn = await aiosqlite.connect(self.dbpath)
+        return self.conn
+
+    async def exec_db (self, func, *args, **kwargs):
+        conn = await self.get_db_conn()
+        cursor = await conn.cursor()
+        try:
+            ret = await func(*args, conn=conn, cursor=cursor, **kwargs)
+        except:
+            await cursor.close()
+            raise
+        await cursor.close()
+        return ret
+
     def _log_exception(self, e, halt=True):
         if halt:
             raise e
@@ -137,7 +173,7 @@ class CravatReport:
 
     async def getjson (self, level):
         ret = None
-        if await self.table_exists(level) == False:
+        if await self.exec_db(self.table_exists, level) == False:
             return ret
         for row in await self.cf.exec_db(self.cf.getiterator, level):
             row = self.substitute_val(level, row)
@@ -237,10 +273,19 @@ class CravatReport:
             new_datarow[colno] = newcell
         return new_datarow
 
+    def get_extracted_header_columns (self, level):
+        cols = []
+        for col in self.colinfo[level]['columns']:
+            if col['col_name'] in self.colnames_to_display[level]:
+                cols.append(col)
+        return cols
+
     async def run_level (self, level):
-        ret = await self.table_exists(level)
+        ret = await self.exec_db(self.table_exists, level)
         if ret == False:
             return
+        if self.should_write_level(level) == False:
+            return
         gene_summary_datas = {}
         if level == 'variant':
             await self.cf.exec_db(self.cf.make_filtered_uid_table)
@@ -276,6 +321,7 @@ class CravatReport:
                                 cats.append(val)
                         self.colinfo[level]['columns'][i]['col_cats'] = cats
         self.write_preface(level)
+        self.extracted_cols[level] = self.get_extracted_header_columns(level)
         self.write_header(level)
         if level == 'variant':
             hugo_present = 'base__hugo' in self.colnos['variant']
@@ -294,6 +340,12 @@ class CravatReport:
             write_variant_sample_separately = False
         colnos = self.colnos[level]
         all_mappings_newcolno = self.newcolnos['variant']['base__all_mappings']
+        cols = self.colinfo[level]['columns']
+        json_colnos = []
+        for i in range(len(cols)):
+            col = cols[i]
+            if col['table'] == True:
+                json_colnos.append(i)
         for datarow in datarows:
             if datarow is None:
                 continue
@@ -359,21 +411,15 @@ class CravatReport:
                     for sample in samples:
                         sample_datarow = new_datarow
                         sample_datarow[sample_newcolno] = sample
-                        self.write_table_row(sample_datarow)
+                        self.write_table_row(self.get_extracted_row(sample_datarow))
                 else:
-                    self.write_table_row(new_datarow)
+                    self.write_table_row(self.get_extracted_row(new_datarow))
             else:
-                self.write_table_row(new_datarow)
+                self.write_table_row(self.get_extracted_row(new_datarow))
 
-    async def get_db_conn (self):
-        if self.dbpath is None:
-            return None
-        conn = await aiosqlite.connect(self.dbpath)
-        return conn
-
-    async def store_mapper (self):
-        conn = await self.get_db_conn()
-        cursor = await conn.cursor()
+    async def store_mapper (self, conn=None, cursor=None):
+        #conn = await self.get_db_conn()
+        #cursor = await conn.cursor()
         q = 'select colval from info where colkey="_mapper"'
         await cursor.execute(q)
         r = await cursor.fetchone()
@@ -381,54 +427,69 @@ class CravatReport:
             self.mapper_name = 'hg38'
         else:
             self.mapper_name = r[0].split(':')[0]
-        await cursor.close()
-        await conn.close()
+        #await cursor.close()
+        #await conn.close()
 
     async def run (self, tab='all'):
-        start_time = time.time()
-        if not (hasattr(self, 'no_log') and self.no_log):
-            self.logger.info('started: %s'%time.asctime(time.localtime(start_time)))
-        if self.module_conf is not None and self.status_writer is not None:
-            if self.parsed_args.do_not_change_status == False:
-                self.status_writer.queue_status_update('status', 'Started {} ({})'.format(self.module_conf['title'], self.module_name))
-        if self.setup() == False:
-            await self.close_db()
-            return
-        if tab == 'all':
-            for level in await self.cf.exec_db(self.cf.get_result_levels):
-                if await self.table_exists(level):
-                    await self.make_col_info(level)
-            for level in await self.cf.exec_db(self.cf.get_result_levels):
-                if await self.table_exists(level):
-                    await self.run_level(level)
-        else:
-            if tab in ['variant', 'gene']:
-                for level in ['variant', 'gene']:
-                    if await self.table_exists(level):
-                        await self.make_col_info(level)
+        try:
+            start_time = time.time()
+            if not (hasattr(self, 'no_log') and self.no_log):
+                self.logger.info('started: %s'%time.asctime(time.localtime(start_time)))
+            if self.module_conf is not None and self.status_writer is not None:
+                if self.parsed_args.do_not_change_status == False:
+                    self.status_writer.queue_status_update('status', 'Started {} ({})'.format(self.module_conf['title'], self.module_name))
+            if self.setup() == False:
+                await self.close_db()
+                return
+            if tab == 'all':
+                for level in await self.cf.exec_db(self.cf.get_result_levels):
+                    self.level = level
+                    if await self.exec_db(self.table_exists, level):
+                        await self.exec_db(self.make_col_info, level)
+                for level in await self.cf.exec_db(self.cf.get_result_levels):
+                    self.level = level
+                    if await self.exec_db(self.table_exists, level):
+                        await self.run_level(level)
             else:
-                await self.make_col_info(tab)
-            await self.run_level(tab)
-        await self.close_db()
-        if self.module_conf is not None and self.status_writer is not None:
-            if self.parsed_args.do_not_change_status == False:
-                self.status_writer.queue_status_update('status', 'Finished {} ({})'.format(self.module_conf['title'], self.module_name))
-        end_time = time.time()
-        if not (hasattr(self, 'no_log') and self.no_log):
-            self.logger.info('finished: {0}'.format(time.asctime(time.localtime(end_time))))
-            run_time = end_time - start_time
-            self.logger.info('runtime: {0:0.3f}'.format(run_time))
-        ret = self.end()
+                if tab in ['variant', 'gene']:
+                    for level in ['variant', 'gene']:
+                        if await self.exec_db(self.table_exists, level):
+                            await self.exec_db(self.make_col_info, level)
+                else:
+                    await self.exec_db(self.make_col_info, tab)
+                self.level = level
+                await self.run_level(tab)
+            await self.close_db()
+            if self.module_conf is not None and self.status_writer is not None:
+                if self.parsed_args.do_not_change_status == False:
+                    self.status_writer.queue_status_update('status', 'Finished {} ({})'.format(self.module_conf['title'], self.module_name))
+            end_time = time.time()
+            if not (hasattr(self, 'no_log') and self.no_log):
+                self.logger.info('finished: {0}'.format(time.asctime(time.localtime(end_time))))
+                run_time = end_time - start_time
+                self.logger.info('runtime: {0:0.3f}'.format(run_time))
+            ret = self.end()
+        except:
+            await self.close_db()
+            if self.module_conf is not None and self.status_writer is not None:
+                if self.parsed_args.do_not_change_status == False:
+                    self.status_writer.queue_status_update('status', 'Failed {} ({})'.format(self.module_conf['title'], self.module_name))
+            end_time = time.time()
+            if not (hasattr(self, 'no_log') and self.no_log):
+                self.logger.info('finished: {0}'.format(time.asctime(time.localtime(end_time))))
+                run_time = end_time - start_time
+                self.logger.info('runtime: {0:0.3f}'.format(run_time))
+            raise
         return ret
 
     async def get_variant_colinfo (self):
         self.setup()
         level = 'variant'
-        if await self.table_exists(level):
-            await self.make_col_info(level)
+        if await self.exec_db(self.table_exists, level):
+            await self.exec_db(self.make_col_info, level)
         level = 'gene'
-        if await self.table_exists(level):
-            await self.make_col_info(level)
+        if await self.exec_db(self.table_exists, level):
+            await self.exec_db(self.make_col_info, level)
         return self.colinfo
 
     def setup (self):
@@ -446,10 +507,38 @@ class CravatReport:
     def write_table_row (self, row):
         pass
 
-    async def make_col_info (self, level):
-        conn = await self.get_db_conn()
-        cursor = await conn.cursor()
-        await self.store_mapper()
+    def get_extracted_row (self, row):
+        if self.display_select_columns[self.level]:
+            filtered_row = [row[colno] for colno in self.colnos_to_display[self.level]]
+        else:
+            filtered_row = row
+        return filtered_row
+
+    def add_conditional_to_colnames_to_display (self, level, column, module_name):
+        col_name = column['col_name']
+        if level in self.extract_columns_multilevel and len(self.extract_columns_multilevel[level]) > 0:
+            if col_name in self.extract_columns_multilevel[level]:
+                incl = True
+            else:
+                incl = False
+        elif self.concise_report:
+            if 'col_hidden' in column and column['col_hidden'] == True:
+                incl = False
+            else:
+                incl = True
+        else:
+            incl = True
+        if incl and col_name not in self.colnames_to_display[level]:
+            if module_name == self.mapper_name:
+                self.colnames_to_display[level].append(col_name.replace(module_name + '__', 'base__'))
+            elif module_name == 'tagsampler':
+                self.colnames_to_display[level].append(col_name.replace(module_name + '__', 'base__'))
+            else:
+                self.colnames_to_display[level].append(col_name)
+
+    async def make_col_info (self, level, conn=None, cursor=None):
+        self.colnames_to_display[level] = []
+        await self.exec_db(self.store_mapper)
         cravat_conf = self.conf.get_cravat_conf()
         if 'report_module_order' in cravat_conf:
             priority_colgroupnames = cravat_conf['report_module_order']
@@ -491,19 +580,18 @@ class CravatReport:
             [colgrpname, _] = coldef.name.split('__')
             column = coldef.get_colinfo()
             columns.append(column)
+            self.add_conditional_to_colnames_to_display(level, column, colgrpname)
             for columngroup in self.columngroups[level]:
                 if columngroup['name'] == colgrpname:
                     columngroup['count'] += 1
         # adds gene level columns to variant level.
-        if self.nogenelevelonvariantlevel == False and level == 'variant' and await self.table_exists('gene'):
+        if self.nogenelevelonvariantlevel == False and level == 'variant' and await self.exec_db(self.table_exists, 'gene'):
             modules_to_add = []
             q = 'select name from gene_annotator'
             await cursor.execute(q)
             gene_annotators = [v[0] for v in await cursor.fetchall()]
             modules_to_add = [m for m in gene_annotators if m != 'base']
             for module in modules_to_add:
-                if not module in gene_annotators:
-                    continue
                 cols = []
                 q = 'select col_def from gene_header where col_name like "{}__%"'.format(module)
                 await cursor.execute(q)
@@ -530,6 +618,7 @@ class CravatReport:
                             coldef.categories.append(r[0])
                     column = coldef.get_colinfo()
                     columns.append(column)
+                    self.add_conditional_to_colnames_to_display(level, column, module)
                     self.var_added_cols.append(coldef.name)
         # Gene level summary columns
         if level == 'gene':
@@ -574,6 +663,7 @@ class CravatReport:
                     coldef.genesummary = True
                     column = coldef.get_colinfo()
                     columns.append(column)
+                    self.add_conditional_to_colnames_to_display(level, column, mi.name)
                 self.summarizing_modules.append([mi, annot, cols])
                 for col in cols:
                     fullname = module_name+'__'+col['name']
@@ -607,6 +697,7 @@ class CravatReport:
         new_columns = []
         self.newcolnos[level] = {}
         newcolno = 0
+        new_colnames_to_display = []
         for colgrp in newcolgrps:
             colgrpname = colgrp['name']
             for col in columns:
@@ -618,17 +709,22 @@ class CravatReport:
                     col['col_name'] = newcolname
                     new_columns.append(col)
                     self.newcolnos[level][newcolname] = newcolno
+                    if newcolname in self.colnames_to_display[level]:
+                        new_colnames_to_display.append(newcolname)
                 elif grpname == colgrpname:
                     new_columns.append(col)
                     self.newcolnos[level][colname] = newcolno
+                    if colname in self.colnames_to_display[level]:
+                        new_colnames_to_display.append(colname)
                 else:
                     continue
                 newcolno += 1
         self.colinfo[level] = {'colgroups': newcolgrps, 'columns': new_columns}
+        self.colnames_to_display[level] = new_colnames_to_display
         # report substitution
         if level in ['variant', 'gene']:
             reportsubtable = level + '_reportsub'
-            if await self.table_exists(reportsubtable):
+            if await self.exec_db(self.table_exists, reportsubtable):
                 q = 'select * from {}'.format(reportsubtable)
                 await cursor.execute(q)
                 reportsub = {r[0]:json.loads(r[1]) for r in await cursor.fetchall()}
@@ -645,8 +741,48 @@ class CravatReport:
                             subs = reportsub[module][col],
                         ))
                         new_columns[i]['reportsub'] = reportsub[module][col]
-        await cursor.close()
-        await conn.close()
+        # display_select_columns
+        if (level in self.extract_columns_multilevel and len(self.extract_columns_multilevel[level]) > 0) or self.concise_report:
+            self.display_select_columns[level] = True
+        else:
+            self.display_select_columns[level] = False
+        # column numbers to display
+        colno = 0
+        self.colnos_to_display[level] = []
+        for colgroup in self.colinfo[level]['colgroups']:
+            count = colgroup['count']
+            if count == 0:
+                continue
+            for col in self.colinfo[level]['columns'][colno:colno+count]:
+                module_col_name = col['col_name']
+                if module_col_name in self.colnames_to_display[level]:
+                    include_col = True
+                else:
+                    include_col = False
+                if include_col:
+                    self.colnos_to_display[level].append(colno)
+                colno += 1
+
+    def get_standardized_module_option (self, v):
+        tv = type(v)
+        if tv == str:
+            if ':' in v:
+                v0 = {}
+                for v1 in v.split('.'):
+                    if ':' in v1:
+                        v1toks = v1.split(':')
+                        if len(v1toks) == 2:
+                            level = v1toks[0]
+                            v2s = v1toks[1].split(',')
+                            v0[level] = v2s
+                v = v0
+            elif ',' in v:
+                v = [val for val in v.split(',') if val != '']
+        if v == 'true':
+            v = True
+        elif v == 'false':
+            v = False
+        return v
 
     async def connect_db (self, dbpath=None):
         if dbpath != None:
@@ -659,6 +795,8 @@ class CravatReport:
             exit()
 
     async def close_db (self):
+        if hasattr(self, 'conn') and self.conn is not None:
+            await self.conn.close()
         if self.cf is not None:
             await self.cf.close_db()
             self.cf = None
@@ -667,9 +805,7 @@ class CravatReport:
         self.cf = await CravatFilter.create(dbpath=self.dbpath)
         await self.cf.exec_db(self.cf.loadfilter, filter=self.filter, filterpath=self.filterpath, filtername=self.filtername, filterstring=self.filterstring)
 
-    async def table_exists (self, tablename):
-        conn = await self.get_db_conn()
-        cursor = await conn.cursor()
+    async def table_exists (self, tablename, conn=None, cursor=None):
         sql = 'select name from sqlite_master where ' + \
             'type="table" and name="' + tablename + '"'
         await cursor.execute(sql)
@@ -678,8 +814,6 @@ class CravatReport:
             ret = False
         else:
             ret = True
-        await cursor.close()
-        await conn.close()
         return ret
 
 def clean_args (cmd_args):
@@ -728,31 +862,63 @@ def run_reporter (*inargs, **inkwargs):
         savedir = os.path.dirname(args.savepath)
         if savedir != '':
             output_dir = savedir
+    module_options = {}
+    if args.module_option is not None:
+        for opt_str in args.module_option:
+            toks = opt_str.split('=')
+            if len(toks) != 2:
+                if not args.silent:
+                    print('Ignoring invalid module option {opt_str}. module-option should be module_name.key=value.')
+                continue
+            k = toks[0]
+            if k.count('.') != 1:
+                if not args.silent:
+                    print('Ignoring invalid module option {opt_str}. module-option should be module_name.key=value.')
+                continue
+            [module_name, key] = k.split('.')
+            if module_name not in module_options:
+                module_options[module_name] = {}
+            v = toks[1]
+            module_options[module_name][key] = v
+    del args.module_option
     loop = asyncio.get_event_loop()
     response = {}
     for report_type in report_types:
         if args.silent == False:
             print(f'Generating {report_type} report... ', end='', flush=True)
         module_info = au.get_local_module_info(report_type + 'reporter')
-        spec = importlib.util.spec_from_file_location(module_info.name, module_info.script_path)
+        module_name = module_info.name
+        spec = importlib.util.spec_from_file_location(module_name, module_info.script_path)
         module = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(module)
-        args.module_name = module_info.name
+        args.module_name = module_name
         args.do_not_change_status = True
+        if module_name in module_options:
+            args.conf = module_options[module_name]
         reporter = module.Reporter(args)
         loop.run_until_complete(reporter.prep())
+        response_t = None
         try:
             response_t = loop.run_until_complete(reporter.run())
+            output_fns = None
             if args.silent == False:
-                print(f'report created in {os.path.abspath(output_dir)}.')
-        except:
+                if type(response_t) == list:
+                    output_fns = ' '.join(response_t)
+                else:
+                    output_fns = response_t
+                if output_fns is not None:
+                    print(f'report created: {output_fns}')
+        except Exception as e:
             if args.silent == False:
                 print(f'report generation failed.')
-            loop.run_until_complete(reporter.close_db())
-            response_t = {'success': False}
-            raise
+            response_t = None
+            import traceback
+            traceback.print_exc()
         response[report_type] = response_t
-    return response
+    if len(report_types) == 1 and len(response) == 1:
+        return response[list(response.keys())[0]]
+    else:
+        return response
 
 def cravat_report_entrypoint ():
     global parser
@@ -830,5 +996,14 @@ parser.add_argument('--system-option',
     dest='system_option',
     nargs='*',
     help='System option in key=value syntax. For example, --system-option modules_dir=/home/user/open-cravat/modules')
+parser.add_argument('--module-option',
+    dest='module_option',
+    nargs='*',
+    help='Module-specific option in module_name.key=value syntax. For example, --module-option vcfreporter.type=separate')
+parser.add_argument('--concise-report',
+    dest='concise_report',
+    action='store_true',
+    default=False,
+    help='Generate concise report with default columns defined by annotation modules')
 parser.set_defaults(func=run_reporter)
 
